// Algo358D.cpp : Defines the entry point for the console application.

//				Tests the CSVD routine, ACM Algorithm 358
//
//		Singular Value Decomposition, a = u * s * Conj(Tran(v)), a is destroyed by CSVD
//		the diagonal matrix s is output as a vector, m must be >= n
//		if smaller, a should be filled with zero rows
//		this code is adapted from Collected Algorithms from ACM, Algorithm 358
//		The transformation Conj(Tran(u)) is applied to the p vectors given in columns
//		n, n+1, ..., n+p-1 of matrix a
//		See: http://www.scs.fsu.edu/~burkardt/f77_src/toms358/toms358.f
//		and: http://www.scs.fsu.edu/~burkardt/f77_src/toms358/toms358_prb.f

#include "stdafx.h"
#include "Algo358.h"
#include <iostream>		// for cout
#include <time.h>		// for _strdate and _strtime
#include <iomanip>		// for setiosflags and setprecision
#define nL	'\n'		// new line
#define dL	"\n\n"		// double new line

using namespace System;
using namespace System::Reflection;		// for Assembly. ...
using namespace std;

void CSVD(complex<double> a[][maxA], int m, int n, int p, int nu, int nv,
		double *s, complex<double> u[][maxA], complex<double> v[][maxA]);
void TestCSVD();
void TimeStamp();
void WriteMatrix(complex<double> [][maxA], int, int);

int ti;		// test case number, 1 or 2
double time0;

// ------- initialize
void init(void)
{	
	char name[]    = "Algo358D - Tests the CSVD routine, ACM Algorithm 358";
	char author1[] = "Copyright (c) 1981-2007 by author: Harry J. Smith,";
	char author2[] = "19628 Via Monte Dr., Saratoga, CA 95070.  All rights reserved.";
	char usage1[]  = "Usage: Algo358D [1, 2] [>Test.out]";
	String^ version;		// = "C++ Version x.x.xxxx.xxxxx"
	Version aVersion = Assembly::GetExecutingAssembly()->GetName()->Version->ToString();
	version = "C++ Console Version " + aVersion.ToString() + " ";
	cout << setiosflags(ios::fixed) << setprecision(4) << "";
	cout << nL << name << nL;	// show credits
	int len;
	len = version->LastIndexOf(" ");
	char sta[40];
	for (int i = 0; i < len; i++)
		sta[i] = (char)version[i];
	sta[len] = 0;
	cout << sta << nL;
	cout << author1 << nL;
	cout << author2 << dL;
	cout << usage1 << dL;		// show usage
} // init

// The main entry point for the application Algo358D
//int _tmain(int argc, _TCHAR* argv[])
int main(int argc, char* argv[])
{
	init();
	char tc[3] = "1";
	if (argc > 1)							// if arguments
		strncpy_s(tc, 3, argv[1], 2);		// Copy string from user input line
	ti = atoi(tc);
	if (ti != 1 && ti != 2)
		ti = 1;
	cout << "Running test case " << ti << " of 2" << dL;
	time0 = Environment::TickCount / 1000.0;
	TimeStamp();
	cout << nL << "Algo358D:" << nL;
	cout << "Test the ACM TOMS 358 Algorithm CSVD" << dL;
	TestCSVD();
	cout << "Algo358D:" << nL;
	cout << "Normal end of execution" << dL;
	TimeStamp();
	return 0;
} // main

void TestCSVD()		//test CSVD
{
	int i, j, k;
	int m = 5;
	int n = 5;
	if (ti == 2)
		n = 4;
	complex <double> a[maxA][maxA];
	double s[maxA];
	complex <double> u[maxA][maxA];
	complex <double> v[maxA][maxA];
	cout << "TestCSVD" << nL;
	cout << "Call ACM TOMS Algorithm 358 for the" << nL;
	cout << "singular value decomposition:" << nL;
	cout << "  A = U * S * V^H" << nL;
	cout << "  of an M by N complex matrix A." << nL;
	a[0][0] = complex<double>( 0.4499, -0.1267);	// Set the matrix
	a[0][1] = complex<double>(-0.2361,  0.0775);
	a[0][2] = complex<double>( 0.5008, -0.7799);
	a[0][3] = complex<double>(-0.7702, -0.3143);
	a[0][4] = complex<double>(-0.5084,  0.5621);
	a[1][0] = complex<double>(-0.8432, -0.3443);
	a[1][1] = complex<double>( 0.0186, -0.6332);
	a[1][2] = complex<double>( 0.3505,  0.0166);
	a[1][3] = complex<double>(-0.8892,  0.2657);
	a[1][4] = complex<double>(-0.5066,  0.6005);
	a[2][0] = complex<double>( 0.5896,  0.2601);
	a[2][1] = complex<double>( 0.8928,  0.0103);
	a[2][2] = complex<double>( 0.4350, -0.2666);
	a[2][3] = complex<double>(-0.7799, -0.5512);
	a[2][4] = complex<double>(-0.1022, -0.4500);
	a[3][0] = complex<double>( 0.3911,  0.3234);
	a[3][1] = complex<double>(-0.5605,  0.7638);
	a[3][2] = complex<double>(-0.2009,  0.2707);
	a[3][3] = complex<double>( 0.0314, -0.4336);
	a[3][4] = complex<double>(-0.1046,  0.3267);
	a[4][0] = complex<double>(-0.1395, -0.1561);
	a[4][1] = complex<double>( 0.3064,  0.0263);
	a[4][2] = complex<double>(-0.0975,  0.9019);
	a[4][3] = complex<double>( 0.2491,  0.5787);
	a[4][4] = complex<double>( 0.4106, -0.8102);
	cout << nL << "Matrix A:" << dL;
	WriteMatrix(a, m, n);
	int p = 0;
	int nu = m;
	int nv = n;
	CSVD(a, m, n, p, nu, nv, s, u, v);
	cout << "Singular values (Diagonal of diagonal S matrix):" << dL;
	for (i = 0; i < n; i++)
		 cout << setprecision(14) << "  " << s[i] << nL;
	cout << setprecision(4) << "";
	if (m > n)
		cout << nL << "U (Augmented to an orthogonal basis for the output space):" << dL;
	else
		cout << nL << "U:" << dL;
	WriteMatrix(u, m, m);
	cout << "V:" << dL;
	WriteMatrix(v, n, n);
	for (i = 0; i < m; i++)		// Compute U * S * V^H
		 for (j = 0; j < n; j++)
		 {
				 a[i][j] = 0.0, 0.0;
				 for (k = 0; k < m && k < n; k++)
						 a[i][j] += u[i][k] * s[k] * conj(v[j][k]);
		 }
	cout << "Matrix U * S * V^H (Should equal the original A):" << dL;
	WriteMatrix(a, m, n);
} // TestCSVD

void WriteMatrix(complex <double> a[][maxA], int m, int n)
// print an m by n complex matrix
{
	for (int i = 0; i < m; i++)
	{
		for (int j = 0; j < n; j++)
		{
			cout << " ";
			if (real(a[i][j]) >= 0.0)
				cout << " ";
			else
				cout << "-";
			cout << abs(real(a[i][j]));
			if (imag(a[i][j]) >= 0.0)
				cout << "+";
			else
				cout << "-";
			cout << abs(imag(a[i][j]));
		}
		cout << nL;
	}
	cout << nL;
} // WriteMatrix

void TimeStamp()
/// printout the current MM/dd/yyyy hh:mm:ss AM/PM  s.ss sec. as a timestamp
{
	double time1 = Environment::TickCount / 1000.0; // TickCount = number of milliseconds since Windows started.
	double timeD = time1 - time0;
	time0 = time1;
	DateTime dt = DateTime::Now;
	String^ st = dt.ToString("MM/dd/yyyy hh:mm:ss tt");
	st += "  " + timeD.ToString("0.00") + " sec.";
	int len;
	len = 5 + st->LastIndexOf(" sec.");
	char sta[79];
	for (int i = 0; i < len; i++)
	{
		sta[i] = (char)st[i];
	}
	sta[len] = 0;
	cout << sta << nL;
} // TimeStamp

/* end of file Algo358D.cpp ----------------------------------------- */
